/**
 * Created by Nikolay Ilin (na.iline@me.com) on 23/01/2019.
 * Copyrights by Soft-Artel LIMITED
 * www.soft-artel.com
 */

const querystring = require('querystring');

module.exports = class SAError {
  // --------------------------------

  constructor(type) {
    this.type = type;
  }

  // --------------------------------

  throw(err, status = null, params = {}, alert = true) {
    if (typeof err === 'string') {
      err = new Error(err);
      err.stack = err.stack.split('\n').slice(1).join('\n');
    }

    status = status || err.status || err.code || err.statusCode || 500;

    if (typeof status !== 'number') {
      err.message = `${JSON.stringify(status)} ${err.message}`;
      status = 502;
    }

    err.status = status;

    if (params && (typeof params !== 'object' || Array.isArray(params))) {
      params = { params };
    }

    if (params) {
      err.params = Object.merge(params, err.params || {});
    }

    err.alert = alert;

    if (err.params) {
      delete err.params.password;

      // api-app
      if (err.params.auth_user) {
        err.params.auth_user = querystring.unescape(err.params.auth_user);
      }

      if (err.params.auth_device) {
        err.params.auth_device = querystring.unescape(err.params.auth_device);
      }

      // delete err.params.auth_user;
      // delete err.params.auth_device;

      //  model
      delete err.params._context;
      delete err.params._user;
      delete err.params._logger;
    }

    err.message = `${this.type.toUpperCase()} ${err.message}`;
    throw (err);
  }
};
