/**
 * Created by Nikolay Ilin (na.iline@me.com) on 14.12.2017.
 * Copyrights by Soft-Artel LIMITED
 * www.soft-artel.com
 */
require('sa-prototype');

const path = require('path');
const os = require('os');

const { promisify } = require('util');
const fs = require('fs');

const writeFile = promisify(fs.writeFile);
const readFile = promisify(fs.readFile);
const appendFile = promisify(fs.appendFile);
const unlink = promisify(fs.unlink);
const readdir = promisify(fs.readdir);
const access = promisify(fs.access);
const mkdir = promisify(fs.mkdir);
const lstat = promisify(fs.lstat);
const copyFile = promisify(fs.copyFile);

//----------------------------------------------------------------------------

async function tryPath(toPath, make = false) {
  if (toPath[0] === '~') {
    toPath = toPath.substr(1, toPath.length - 1);
    toPath = os.homedir() + toPath;
  }

  toPath = path.normalize(toPath);

  try {
    await access(toPath);

    return true;
  } catch (err) {
    if (make) {
      const pathInfo = path.parse(toPath);

      if (pathInfo.dir !== pathInfo.root && await tryPath(pathInfo.dir, true)) {
        if (!pathInfo.ext || pathInfo.ext === '') {
          await mkdir(toPath);
        }

        return true;
      }
    }

    return false;
  }
}

module.exports = {
  writeFile,
  readFile,
  appendFile,
  unlink,
  readdir,
  access,
  mkdir,
  lstat,
  copyFile,
  tryPath,
  constants: fs.constants,
  createReadStream: fs.createReadStream,
  createWriteStream: fs.createWriteStream
};
